SystemOrganization addCategory: #'Seaside-Adaptors-Swazoo'!

WAServerAdaptor subclass: #WASwazooAdaptor
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Swazoo'!

!WASwazooAdaptor class methodsFor: 'testing' stamp: 'lr 2/15/2009 18:01'!
isAbstract
	^ self name = #WABaseServerAdaptor! !

!WASwazooAdaptor class methodsFor: 'actions' stamp: 'jf 2/15/2009 23:03'!
resetAll
	"removes all Seaside sites, run this if you run into trouble creating servers"
	| server toRemove |
	server := SwazooServer singleton.
	toRemove := server allSites select: [ :each |
		each isKindOf: WAPluggableSite ].
	toRemove do: [ :each |
		server removeSite: each ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'jf 8/31/2009 16:32'!
basicStart
	site port: self port.
	SwazooServer singleton addSite: site.
	site start! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'jf 8/31/2009 16:32'!
basicStop
	SwazooServer singleton removeSite: site! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/16/2009 20:57'!
convertCookiesFrom: aWAResponse to: aSwazooResponse
	| cookies setCookieField setCookie2Field |
	cookies := aWAResponse cookies.
	cookies isEmpty ifTrue: [ ^self ].
	
	setCookieField := WorkingHTTPSetCookieField new.
	setCookie2Field := WorkingHTTPSetCookie2Field new.

	cookies do: [ :each |
		setCookieField addCookie: each oldNetscapeString.
		setCookie2Field addCookie: each rfc2965String ].
	
	aSwazooResponse headers addField: setCookieField.
	aSwazooResponse headers addField: setCookie2Field! !

!WASwazooAdaptor methodsFor: 'initialization' stamp: 'jf 8/31/2009 19:10'!
initialize
	super initialize.
	site := WAPluggableSite
		named: (WAKeyGenerator current keyOfLength: 32)
		on: self.
	site ip: '*'.
	site host: '*'! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 18:02'!
isRunning
	^ site isServing! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 17:47'!
isStarting
	^ false! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 18:02'!
isStopped
	^ site isServing not! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 18:02'!
isStopping
	^ false! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'jf 2/15/2010 20:39'!
postFieldsOf: aNativeRequest into: aRequestFields
	"break encapsulation - This line is likely unnecessary because #requestBodyFor:
		will have already done this after extracting the body."
	aNativeRequest ensureFullRead. ""
	aNativeRequest postData keysAndValuesDo: [ :key :value |
		| previousData data |
		data := value filename isEmptyOrNil
			ifTrue: [ self codec url decode: value value ]
			ifFalse: [
				| isoCodec |
				isoCodec := GRCodec forEncoding: 'ISO-8859-1'.
				WAFile new
					fileName: (self codec url decode: value filename);
					contentType: value contentType seasideMimeType;
					"does a 1:1 mapping from character values to bytes"
					contents: (isoCodec encode: value value) asByteArray;
					yourself ].
		aRequestFields at: key add: data ]! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'lr 2/16/2009 20:49'!
requestAddressFor: aNativeRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aNativeRequest peer.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'jf 2/15/2010 20:28'!
requestBodyFor: aNativeRequest
	"This only works for POST and PUT requests... LAME!!"

	aNativeRequest isPost
		ifTrue: [
			| body |
			"Nasty nastiness but we need to get the raw body before Swazoo parses
			the POST data. Then we need to manually reinject the data to allow it to
			be parsed (since the data has already been pulled off the socket)."
			body := aNativeRequest postData stream nextBytes: aNativeRequest contentLength.
			aNativeRequest parsePostDataFrom: (SwazooStream on: body).
			aNativeRequest postData setParsed.
			^ body ].
	aNativeRequest isPut
		ifTrue: [ ^aNativeRequest putData ].
	^nil! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'lr 2/16/2009 20:49'!
requestCookiesFor: aNativeRequest
	^ WARequestCookie fromString: aNativeRequest cookie! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'jf 2/15/2010 18:02'!
requestFieldsFor: aNativeRequest
	| fields |
	fields := WARequestFields new.
	aNativeRequest isPost
		ifTrue: [ self postFieldsOf: aNativeRequest into: fields ].
	^ fields! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'lr 2/16/2009 20:50'!
requestHeadersFor: aNativeRequest
	| swazooFields fields |
	swazooFields := aNativeRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'lr 2/16/2009 20:50'!
requestMethodFor: aNativeRequest
	^ aNativeRequest methodName! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'jf 1/30/2010 17:43'!
requestUrlFor: aNativeRequest
	| url |
	url := ((WAUrl absolute: aNativeRequest uri printString)
		decodedWith: self codec).
	aNativeRequest isEncrypted
		ifTrue: [ url scheme: 'https' ].
	^ url! !

!WASwazooAdaptor methodsFor: 'converting-request' stamp: 'jf 7/31/2009 11:34'!
requestVersionFor: aNativeRequest
	| array |
	array := aNativeRequest httpVersion.
	^ WAHttpVersion major: array first minor: array second! !

!WASwazooAdaptor methodsFor: 'converting' stamp: 'jf 9/30/2009 01:10'!
responseFrom: aRequestContext 
	| swazooResponse |
	swazooResponse := HTTPResponse new.
	aRequestContext response headers keysAndValuesDo: 
		[ :key :value | 
		key = 'Content-Type' 
			ifTrue: [ swazooResponse contentType: value greaseString ]
			ifFalse: 
				[ | field fieldClass |
				fieldClass := HeaderField classForFieldName: key.
				field := fieldClass 
					newForFieldName: key
					withValueFrom: value greaseString.
				swazooResponse headers addField: field ] ].
	self 
		convertCookiesFrom: aRequestContext response
		to: swazooResponse.
	swazooResponse code: aRequestContext response status asNumber.
	swazooResponse entity: aRequestContext response contents.
	^ swazooResponse! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/8/2008 12:46'!
shutDown
	SwazooServer stop! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/15/2008 10:45'!
startUp
	SwazooServer start! !

!GRPackage class methodsFor: '*seaside-adaptors-swazoo' stamp: 'lr 2/17/2010 15:31'!
seasideAdaptorsSwazoo
	^ self new
		name: 'Seaside-Adaptors-Swazoo';
		description: 'Makes Seaside run on the Swazoo Web server.';
		addDependency: 'Seaside-Core';
		addDependency: 'Seaside-Swazoo';
		url: #seasideUrl;
		yourself! !
